Package: lmviz
Type: Package
Title: A Package to Visualize Linear Models Features and Play with Them
Version: 0.2.0
Author: Francesco Pauli (see file LICENSEMEDIA for credits on sounds and images)
Maintainer: Francesco Pauli <francesco.pauli@deams.units.it>
Description: Contains a suite of shiny applications
    meant to explore linear model inference feature through simulation
    and games.
License: GPL-3
Encoding: UTF-8
LazyData: true
Imports: shiny, shinyjs, lmtest, mgcv, methods, MASS, scatterplot3d,
        rgl, car
RoxygenNote: 7.1.0
NeedsCompilation: no
Packaged: 2020-08-24 13:09:43 UTC; franc
Repository: CRAN
Date/Publication: 2020-08-24 20:40:02 UTC
Built: R 4.6.0; ; 2026-04-25 14:04:45 UTC; unix
