# Generated by roxygen2: do not edit by hand

export(BadLM)
export(Collin)
export(ComputerDecision.default)
export(ConfInt)
export(GuessTheLine)
export(GuessThePoints)
export(LMBoard)
export(MultipleLM)
export(Prediction)
export(QuizResidual)
export(SampleDist)
export(SimpleLM)
export(Simulation.default)
export(SlopeTest)
export(checksim)
import(mgcv)
import(shiny)
importFrom(MASS,mvrnorm)
importFrom(car,ellipse)
importFrom(car,scatter3d)
importFrom(lmtest,bptest)
importFrom(lmtest,resettest)
importFrom(methods,is)
importFrom(rgl,rgl.open)
importFrom(rgl,rglwidget)
importFrom(scatterplot3d,scatterplot3d)
importFrom(stats,dbeta)
importFrom(stats,dchisq)
importFrom(stats,dnorm)
importFrom(stats,dt)
importFrom(stats,lm)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rchisq)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,shapiro.test)
