<!-- README.md is generated from README.Rmd. Please edit that file -->
lmviz
=====

lmviz contains three shiny apps: two are meant to explore linear model inference feature through simulation; the third is a game to learn interpreting diagnostic plots.

Installation
------------

lmviz is available on CRAN, install with

``` r
install.packages("lmviz")
```

Example
-------

The shiny apps are called by the following R functions

``` r
## basic example code
library(lmviz)
SimpleLM()
BadLM()
QuizResidual()
```
